from typing import Iterable
import fhirclient.models.condition as condition
import fhirclient.models.encounter as encounter
import fhirclient.models.fhirreference as fhirreference
import fhirclient.models.patient as patient

"""
FHIR resource classes, capturing a subset of the FHIR specification.

This package contains extensions of the collection of Python classes
 available at https://github.com/smart-on-fhir/client-py
"""


class Condition(condition.Condition):
    """ Detailed information about conditions, problems or diagnoses.

    Extends the Condition definition from
        https://github.com/smart-on-fhir/client-py with:
    - encounter
    """

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.encounter = None
        """ The Encounter during which this Condition was created
        or to which the creation of this record is tightly associated.
        Type `Reference` (represented as `dict` in JSON). """

        super(Condition, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Condition, self).elementProperties()
        js.extend([
            ("encounter", "encounter", fhirreference.FHIRReference, False, None, False),
        ])
        return js


class Collection:
    """
    FHIR data collection, containing Patient, Encounter and Condition resources
    """
    def __init__(self,
                 patients: Iterable[patient.Patient],
                 encounters: Iterable[encounter.Encounter],
                 conditions: Iterable[Condition]):
        self.patients = patients
        self.encounters = encounters
        self.conditions = conditions



#!/usr/bin/env python

import json
import os
import sys
from os import path

import click

from .fhir_reader import FhirReader
from .mapper import Mapper
from transmart_loader.console import Console
from transmart_loader.copy_writer import TransmartCopyWriter
from transmart_loader.loader_exception import LoaderException


@click.command()
@click.argument('input_path')
@click.argument('output_dir')
def fhir2transmart(input_path, output_dir):
    Console.title('FHIR to TranSMART')
    try:
        Console.info('Writing files to {}'.format(output_dir))
        writer = TransmartCopyWriter(output_dir)
        if path.isdir(input_path):
            filenames = [path.join(input_path, filename) for filename in os.listdir(input_path) if filename.endswith('.json')]
        else:
            filenames = [input_path]
        for filename in filenames:
            Console.info('Reading JSON from {}'.format(filename))
            with open(filename, 'r') as input_file:
                data = json.load(input_file)
                collection = FhirReader.read(data)
                result = Mapper.map(collection)
                writer.write_collection(result)
        Console.info('Done.')
    except LoaderException as e:
        Console.error(e)
        sys.exit(1)


def main():
    fhir2transmart()


if __name__ == '__main__':
    main()
